@@ -31,7 +31,7 @@ from nb2workflow.logging_setup import setup_logging
 from nb2workflow.json import CustomJSONEncoder
 
 from nb2workflow.semantics import understand_comment_references, oda_ontology_prefix
-
+from git import Repo, InvalidGitRepositoryError, GitCommandError
 import logging
 
 logger=logging.getLogger(__name__)
@@ -62,6 +62,18 @@ def cast_parameter(x,par):
             return True
         else:
             raise ValueError(f'Parameter {par["name"]} value "{x}" can not be interpreted as boolean.')
+    if par['python_type'] in [list, dict]:
+        try:
+            if type(x) is str:
+                decoded = json.loads(x)
+            else:
+                decoded = x
+            if type(decoded) is par['python_type']:
+                return decoded
+            else:
+                raise ValueError
+        except:
+            raise ValueError(f'Parameter {par["name"]} value "{x}" can not be interpreted as {par["python_type"].__name__}.')
     return par['python_type'](x)
 
 
@@ -108,7 +120,7 @@ def parse_nbline(line: str, nb_uri=None) -> Optional[dict]:
             value = value_str
             python_type = str
             
-        parsed_comment = understand_comment_references(comment, fallback_type=owl_type_for_python_type(python_type))        
+        parsed_comment = understand_comment_references(comment, fallback_type=odahub_type_for_python_type(python_type))
         
         logger.info("parameter name=%s value=%s python_type=%s, owl_type=%s extra_ttl=%s", 
                     name, value, python_type, parsed_comment['owl_type'], parsed_comment['extra_ttl'])
@@ -123,8 +135,56 @@ def parse_nbline(line: str, nb_uri=None) -> Optional[dict]:
                 )
 
 
+def odahub_type_for_python_type(python_type: type):
+    out_type = python_type.__name__
+
+    xml_scheme_url = "http://www.w3.org/2001/XMLSchema#"
+    oda_ontology_url = "http://odahub.io/ontology#"
+
+    if python_type == int:
+        out_type = 'Integer'
+        url_prefix = oda_ontology_url
+    elif python_type == str:
+        out_type = 'String'
+        url_prefix = oda_ontology_url
+    elif python_type == bool:
+        out_type = 'Boolean'
+        url_prefix = oda_ontology_url
+    elif python_type == float:
+        out_type = 'Float'
+        url_prefix = oda_ontology_url
+    else:
+        url_prefix = xml_scheme_url
+
+    output_url = f"{url_prefix}{out_type}"
+
+    return output_url
+
+
 def owl_type_for_python_type(python_type: type):
-    return "http://www.w3.org/2001/XMLSchema#"+ python_type.__name__ 
+    out_type = python_type.__name__
+
+    xml_scheme_url = "http://www.w3.org/2001/XMLSchema#"
+    oda_ontology_url = "http://odahub.io/ontology#"
+
+    if python_type == int:
+        out_type = 'integer'
+        url_prefix = xml_scheme_url
+    elif python_type == str:
+        out_type = 'string'
+        url_prefix = xml_scheme_url
+    elif python_type == bool:
+        out_type = 'boolean'
+        url_prefix = xml_scheme_url
+    elif python_type == float:
+        out_type = 'float'
+        url_prefix = xml_scheme_url
+    else:
+        url_prefix = oda_ontology_url
+
+    output_url = f"{url_prefix}{out_type}"
+
+    return output_url
 
 class InputParameter:
     raw_line=None 
@@ -178,18 +238,23 @@ class InputParameter:
 class NotebookAdapter:
     limit_output_attachment_file = None
 
-    def __init__(self, notebook_fn):
+    def __init__(self, notebook_fn, tempdir_cache=None):
         self.notebook_fn = os.path.abspath(notebook_fn)
         self.name = notebook_short_name(notebook_fn)
+        self.tempdir_cache = tempdir_cache
         logger.debug("notebook adapter for %s", self.notebook_fn)
         logger.debug(self.extract_parameters())
 
-    def new_tmpdir(self):
+    def new_tmpdir(self, cache_key=None):
         logger.debug("tmpdir was "+getattr(self,'_tmpdir','unset'))
         self._tmpdir = None
         logger.debug("tmpdir became %s", self._tmpdir)
 
-        return self.tmpdir
+        newdir = self.tmpdir
+        if ( self.tempdir_cache is not None ) and ( cache_key is not None ):
+            self.tempdir_cache[cache_key] = newdir
+
+        return newdir
 
     @property
     def tmpdir(self):
@@ -347,12 +412,12 @@ class NotebookAdapter:
 
         
 
-    def execute(self, parameters, progress_bar = True, log_output = True, inplace=False):
+    def execute(self, parameters, progress_bar = True, log_output = True, inplace=False, tmpdir_key=None, callback_url=None):
         t0 = time.time()
         logstasher.log(dict(origin="nb2workflow.execute", event="starting", parameters=parameters, workflow_name=notebook_short_name(self.notebook_fn), health=current_health()))
 
         logger.info("starting job")
-        exceptions = self._execute(parameters, progress_bar, log_output, inplace)
+        exceptions = self._execute(parameters, progress_bar, log_output, inplace, callback_url=callback_url, tmpdir_key=tmpdir_key)
             
         tspent = time.time() - t0
         logstasher.log(dict(origin="nb2workflow.execute", 
@@ -365,27 +430,33 @@ class NotebookAdapter:
 
         return exceptions
 
-    def _execute(self, parameters, progress_bar = True, log_output = True, inplace=False):
-
+    def _execute(self, parameters, progress_bar = True, log_output = True, inplace=False, callback_url=None, tmpdir_key=None):
         if not inplace :
-            tmpdir = self.new_tmpdir()
+            tmpdir = self.new_tmpdir(tmpdir_key)
             logger.info("new tmpdir: %s", tmpdir)
-
+            repo_dir = os.path.dirname(os.path.realpath(self.notebook_fn))
             try:
-                output = subprocess.check_output(["git","clone", "--recurse-submodules", os.path.dirname(os.path.realpath(self.notebook_fn)), tmpdir])
-                # output = subprocess.check_output(["git","clone", "--depth", "1", "file://" + os.path.dirname(os.path.realpath(self.notebook_fn)), tmpdir])
-                logger.info("git clone output: %s", output)
-            except Exception as e:
-                logger.warning("git clone failed: %s, will attempt copytree", e)
-
+                repo = Repo(repo_dir)
+                repo.clone(tmpdir, multi_options=["--recurse-submodules"])
+            except InvalidGitRepositoryError:
+                logger.warning(f"repository {repo_dir} is invalid, will attempt copytree")
                 os.rmdir(tmpdir)
-
                 shutil.copytree(os.path.dirname(os.path.realpath(self.notebook_fn)), tmpdir)
+            except GitCommandError as e:
+                logger.warning(f"git command error: {e}")
+                if 'git-lfs' in str(e):
+                    # this error may occur if the repo was originally cloned by the different version of git utility
+                    # e.g. when repo is mounted with docker run -v
+                    raise Exception("We got some problem cloning the repository, the problem seems to be related to git-lfs. You might want to try reinitializing git-lfs with 'git-lfs install; git-lfs pull'")
+                else:
+                    raise e
         else:
             tmpdir =os.path.dirname(os.path.realpath(self.notebook_fn))
             logger.info("executing inplace, no tmpdir is input dir: %s", tmpdir)
 
-        
+        if callback_url:
+            self._pass_callback_url(tmpdir, callback_url)
+
         self.update_summary(state="started", parameters=parameters)
 
         self.inject_output_gathering()
@@ -427,6 +498,17 @@ class NotebookAdapter:
 
         return exceptions
 
+    def _pass_callback_url(self, workdir: str, callback_url: str):
+        """
+        save callback_url to file .oda_api_callback in the notebook dir where it can be accessed by ODA API
+        :param workdir: directory to save notebook in
+        """
+        callback_file = ".oda_api_callback"  # perhaps it would be better to define this constant in a common lib
+        callback_file_path = os.path.join(workdir, callback_file)
+        with open(callback_file_path, 'wt') as output:
+            print(callback_url, file=output)
+        logger.info("callback file created: %s", callback_file_path)
+
     def extract_pm_output(self):
         nb = sb.read_notebook(self.output_notebook_fn)
 
@@ -496,8 +578,9 @@ if isinstance({output},str) and os.path.exists({output}):
         sb.glue(variable_name + "_content", encoded)
     else:
         # TODO: make a customizable upload to different DL platforms; before that it should be enabled with caution    
-        os.makedirs("/tmp/nb2w-store", exist_ok=True)
-        url = "file:///tmp/nb2w-store/" + str(hashlib.md5(content).hexdigest())
+        nb2w_store_base = os.getenv("NB2W_CACHE", os.getenv("HOME") + "/.cache/nb2workflow/bigoutputs")
+        os.makedirs(nb2w_store_base, exist_ok=True)
+        url = "file://" + nb2w_store_base +  "/" + str(hashlib.md5(content).hexdigest())
         print("storing file to URL", url)
         with open(url.replace("file://", ""), "wb") as f:
             f.write(content)
@@ -557,9 +640,13 @@ if isinstance({output},str) and os.path.exists({output}):
 def notebook_short_name(ipynb_fn):
     return os.path.basename(ipynb_fn).replace(".ipynb","")
 
-def find_notebooks(source, tests=False) -> Dict[str, NotebookAdapter]:
+def find_notebooks(source, tests=False, pattern = r'.*') -> Dict[str, NotebookAdapter]:
 
-    base_filter = lambda fn: "output" not in fn and "preproc" not in fn
+    def base_filter(fn): 
+        good = "output" not in fn and "preproc" not in fn
+        good = good and re.match(pattern, os.path.basename(fn)) 
+        return good
+        
 
     if tests:
         filt = lambda fn: base_filter(fn) and "/test_" in fn
@@ -567,7 +654,7 @@ def find_notebooks(source, tests=False) -> Dict[str, NotebookAdapter]:
         filt = lambda fn: base_filter(fn) and "/test_" not in fn
 
     if os.path.isdir(source):
-        notebooks=[ fn for fn in glob.glob(source+"/*ipynb") if filt(fn) ]
+        notebooks=[ fn for fn in glob.glob(source+"/*ipynb") if filt(fn) ]        
 
         logger.debug("found notebooks: %s",notebooks)
 
@@ -581,6 +668,8 @@ def find_notebooks(source, tests=False) -> Dict[str, NotebookAdapter]:
 
 
     elif os.path.isfile(source):
+        if pattern != r'.*':
+            logger.warning('Filename pattern is set but source %s is a single file. Ignoring pattern.')
         notebook_adapters={notebook_short_name(source): NotebookAdapter(source)}
 
     else:
