@@ -103,6 +103,7 @@ def create_app():
 app = create_app()
 
 app.async_workflows = dict()
+app.async_workflow_jobdirs = dict()
 app.started_at = datetime.datetime.now()
 
 
@@ -167,15 +168,15 @@ class AsyncWorkflow:
             async_queue.put(self)
             app.async_workflows[self.key] = 'submitted'
             return
-
-        app.async_workflows[self.key] = 'started'
-
+       
         template_nba = app.notebook_adapters.get(self.target)
-
-        nba = NotebookAdapter(template_nba.notebook_fn)
+        nba = NotebookAdapter(template_nba.notebook_fn, tempdir_cache=app.async_workflow_jobdirs)
+        
+        app.async_workflows[self.key] = 'started'
+        self.perform_callback(action='progress')
 
         try:
-            exceptions = nba.execute(self.params['request_parameters'])
+            exceptions = nba.execute(self.params['request_parameters'], callback_url=self.callback, tmpdir_key=self.key)
         except PapermillWorkflowIncomplete as e:
             logger.info("found incomplete workflow: %s, rescheduling", repr(e))
 
@@ -222,7 +223,7 @@ class AsyncWorkflow:
 
         self.perform_callback()
 
-    def perform_callback(self):
+    def perform_callback(self, action='done'):
         if self.callback is None:
             logger.info('no callback registered, skipping')
             return
@@ -233,7 +234,7 @@ class AsyncWorkflow:
         result = app.async_workflows[self.key]
 
         callback_payload = dict(
-            action='done'
+            action=action
         )
         
         if re.match('^file://', self.callback):
@@ -286,19 +287,34 @@ def workflow(target, background=False, async_request=False):
         print('cache key/value', key, value)
 
         if value is None:
-            async_task = AsyncWorkflow(
-                key=key, target=target, params=interpreted_parameters, callback=async_request_callback)
-
-            async_queue.put(async_task)
+            async_task = AsyncWorkflow(key=key,
+                                       target=target,
+                                       params=interpreted_parameters,
+                                       callback=async_request_callback)
 
             app.async_workflows[key] = 'submitted'
-            return make_response(jsonify(workflow_status="submitted", comment="task created"), 201)
+            async_queue.put(async_task)
 
-        elif value in ['started', 'submitted']:
-            return make_response(jsonify(workflow_status=value, comment="task is "+value), 201)
+            return make_response(jsonify(workflow_status="submitted",
+                                         comment="task created"),
+                                 201)
+
+        elif value == 'submitted':
+            return make_response(jsonify(workflow_status=value,
+                                         comment="task is "+value),
+                                 201)
+
+        elif value == 'started':
+            return make_response(jsonify(workflow_status=value,
+                                         comment="task is "+value,
+                                         jobdir=app.async_workflow_jobdirs.get(key)),
+                                 201)
 
         else:
-            return make_response(jsonify(workflow_status="done", data=value, comment=""), 200)
+            return make_response(jsonify(workflow_status="done",
+                                         data=value,
+                                         comment=""),
+                                 200)
 
     if len(issues) > 0:
         return make_response(jsonify(issues=issues), 400)
@@ -646,7 +662,7 @@ def root():
     if len(issues) == 0:
         return {
                     "message": "all is ok!",
-                    "versiom": os.getenv("ODA_WORKFLOW_VERSION"),
+                    "version": os.getenv("ODA_WORKFLOW_VERSION"),
                     "last_author": os.getenv("ODA_WORKFLOW_LAST_AUTHOR"),
                     "last_changed": os.getenv("ODA_WORKFLOW_LAST_CHANGED")                    
             }
@@ -672,6 +688,7 @@ def main():
         '--profile', metavar='service profile', type=str, default="oda")
     parser.add_argument('--debug', action="store_true")
     parser.add_argument('--one-shot', metavar='workflow', type=str)
+    parser.add_argument('--pattern', type=str, default=r'.*')
 
     args = parser.parse_args()
 
@@ -693,7 +710,7 @@ def main():
         root.setLevel(logging.INFO)
         handler.setLevel(logging.INFO)
 
-    app.notebook_adapters = find_notebooks(args.notebook)
+    app.notebook_adapters = find_notebooks(args.notebook, pattern=args.pattern)
     setup_routes(app)
     app.service_semantic_signature = ontology.service_semantic_signature(
         app.notebook_adapters)
