@@ -44,7 +44,7 @@ def test_nbadapter(test_notebook, morph_notebook, caplog):
     assert 'comment' in parameters['scwid']
     assert parameters['scwid']['owl_type'] == "http://odahub.io/ontology/integral#ScWID"
 
-    assert parameters['enabled']['owl_type'] == "http://www.w3.org/2001/XMLSchema#bool"
+    assert parameters['enabled']['owl_type'] == "http://odahub.io/ontology#Boolean"
 
     outputs = nba.extract_output_declarations()
     print("outputs", outputs)
@@ -71,6 +71,25 @@ def test_nbadapter(test_notebook, morph_notebook, caplog):
 
     assert 'spectrum' in output
 
+def test_find_notebooks(caplog):
+    from nb2workflow.nbadapter import find_notebooks, NotebookAdapter
+    
+    nb_dir = 'tests/testfiles'
+    single_nb = 'tests/testfiles/lightcurve.ipynb'
+    
+    nbas = find_notebooks(single_nb)
+    assert len(nbas) == 1
+    
+    nbas = find_notebooks(single_nb, pattern=r'.*bool')
+    assert len(nbas) == 1
+    assert 'Ignoring pattern.' in caplog.text
+    
+    nbas = find_notebooks(nb_dir)
+    assert len(nbas) == 4
+    
+    nbas = find_notebooks(nb_dir, pattern=r'.*bool')
+    assert len(nbas) == 1
+    
 
 def test_nbadapter_repo(test_notebook_repo):
     from nb2workflow.nbadapter import NotebookAdapter, find_notebooks, validate_oda_dispatcher
@@ -104,6 +123,29 @@ def test_nbadapter_repo(test_notebook_repo):
 
         validate_oda_dispatcher(nba)
 
+@pytest.mark.skip(reason="Reproducing this condition in the test is difficult")
+def test_nbadapter_lfs_repo(test_notebook_lfs_repo):
+    from nb2workflow.nbadapter import NotebookAdapter, find_notebooks, validate_oda_dispatcher
+
+    nbas = find_notebooks(test_notebook_lfs_repo)
+
+    assert len(nbas) >= 1
+
+    for nba_name, nba in nbas.items():
+        print("notebook", nba_name)
+
+        if os.path.exists(nba.output_notebook_fn):
+            os.remove(nba.output_notebook_fn)
+
+        if os.path.exists(nba.preproc_notebook_fn):
+            os.remove(nba.preproc_notebook_fn)
+
+        try:
+            nba.execute(dict())
+            assert False, 'nba.execute is expected to fail'
+        except Exception as ex:
+            assert ex.message == "git-lfs is not initialized"
+        break
 
 def test_nbreduce(test_notebook):
     from nb2workflow.nbadapter import NotebookAdapter, nbreduce, setup_logging
