@@ -125,8 +125,8 @@ def test_service_async_repo(client):
 
 
     while True:
-        if os.path.exists(callback_fn):
-            callback_json = json.load(open(callback_fn))
+        # if os.path.exists(callback_fn):
+        #     callback_json = json.load(open(callback_fn))
             # assert callback_json['action'] == 'done'
         
         options = client.get('/api/v1.0/options')
@@ -141,6 +141,10 @@ def test_service_async_repo(client):
 
         logger.info('service returns %s %s', r, r.json)
 
+        if r.json['workflow_status'] == 'started':
+            assert 'jobdir' in r.json
+            logger.info('jobdir is reported as %s', r.json['jobdir'])
+        
         if r.json['workflow_status'] == 'done':
             logger.info('workflow done!')
             break
